import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

/**
 * Lab #7 CS 2334, Section 0?? DATE GOES HERE
 * <P>
 * This class represents the Model for the MVC lab.
 * It contains the data that will be shared among various views
 * and the method to manipulate the data.
 * </P>
 * 
 * @version 1.0
 */

public class Model {
	// The list of action listeners

	
	// The roster of athletes created from the file

	
	// The roster for a single event

	
	/** The default (no-arg) constructor */
	Model() {
		nationalRoster = null;
		eventRoster = new ArrayList<OlympicAthlete>();
	}

	/**
	 * The constructor for the class.
	 * <P>
	 * 
	 * @param      addressBook 	a list of Contact objects
	 */
	Model(List<OlympicAthlete> nationalRoster) {
		//set the address book
		this.nationalRoster = nationalRoster;

		//create the sublist
		this.eventRoster = new ArrayList<OlympicAthlete>();
	}

	/**
	 *  @return            Returns the national roster.  
	 */
	public List<OlympicAthlete> getNationalRoster() {
		return nationalRoster;
	}

	/**
	 *  @return            Returns the event roster.  
	 */
	public List<OlympicAthlete> getEventRoster() {
		return eventRoster;
	}

	/**
	 * Method to set the subList.
	 * <P>
	 * 
	 * @param      index 	index of the contact selected in the list
	 */
	public void addToEventTeam(int index) {
		eventRoster.add(nationalRoster.get(index));

		// Notify the listener of the change in subList

		
	}

	/** Register an action event listener */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null) {
			actionListenerList = new ArrayList<ActionListener>();
		}
		actionListenerList.add(l);
	}

	/** Remove an action event listener */
	public synchronized void removeActionListener(ActionListener l) {
		if (actionListenerList != null && actionListenerList.contains(l)) {
			actionListenerList.remove(l);
		}
	}

	/** Fire Event */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;
		synchronized (this) {
			if (actionListenerList == null)
				return;
			// Do not worry about the cast warning here.
			list = (ArrayList<ActionListener>) actionListenerList.clone();
		}
		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}

	/**
	 * This method reads in a specified comma delimited file. 
	 * The file is read one line at a time. The read line is then
	 * split and a OlympicAthlete object is created using the parsed
	 * line.
	 * The OlympicAthlete object is then added to a List, which is returned
	 * when the entire file has been processed.
	 * <P>
	 * 
	 * @param      filename     String 		name of the file to be read
	 * @return            		Returns a List of OlympicAthlete objects  
	 */

	public void readFile(String filename) throws FileNotFoundException,
			IOException {
		// Create a file object
		File file = new File(filename);

		// Check if a specified file exists, if not then exit the program
		if (!file.exists()) {
			System.out.println("ERROR:: File not found.");
			System.exit(0);
		}

		// Create file reader and buffered reader to read the file
		FileReader fileReader = new FileReader(filename);
		BufferedReader br = new BufferedReader(fileReader);

		List<OlympicAthlete> nationalRoster = new ArrayList<OlympicAthlete>();

		//read a line form the file, if the first is empty exit the program 
		String line = br.readLine();
		if (line.length() <= 0) {
			System.out.println("ERROR:: There is no data in file.");
			System.exit(0);
		}

		//read file one line at a time, until the end f file
		do {
			OlympicAthlete athlete = new OlympicAthlete(line.split(","));
			nationalRoster.add(athlete);
			line = br.readLine();
		} while (line != null);

		// Notify the listener of the change in addressBook
		this.nationalRoster = nationalRoster;
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
				"National Roster"));
	}
}